package com.utils
{
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import com.greensock.TweenLite;
	import flash.geom.Rectangle;
	import flash.utils.setInterval;
	import flash.utils.clearInterval;
	
	/**
	 * 
	 * @author Iargo
	 */
	public class ScrollVerticalFB extends MovieClip
	{
		private var scroll:mcScrollVertical;
		private var _mascara:Sprite;
		private var _container:Sprite;
		private var _movimento:Number;
		private var _redimensionamento:Boolean;
		private var _delay:Number;
		private var _botoes:Boolean;
		private var _carrinho:Boolean;
		
		private var limiteArraste:Rectangle;
		
		private var pontoMaximo:Number;
		private var pontoMinimo:Number;
		
		private var destinoContainer:Number;
		private var destinoCarrinho:Number;
		
		private var intervaloBotoes:uint;
		
		/**
		 * O Construtor da classe ScrollVerticalFB.
		 * 
		 * <p/>
		 * 
		 * Cria um scroll vertical ao lado do movie clip mascara
		 * 
		 * <p/>
		 * 
		 * 
		 * 
		 * @param	mascara					Identifica o Sprite usado como mascara para a exibição da imagem
		 * 						
		 * @param	container				Identifica o Sprite onde está o conteudo a ser exibido somente no espaço determinado pela máscara
		 * 
		 * @param	movimento				Espaço que o container vai se movimentar a cada clique nos botões cima_mc e baixo_mc
		 * 
		 * @param	redimensionamento		Redimensiona ou não o carrinho_mc (MovieClip arrastável) de acordo com a proporção mascara/container
		 *  
		 * @param	delay					Tempo para que o container chegue ao local de destino após um clique ou após o usuário arrastar o carrinho_mc
		 * 
		 * @param	botoes					"true" para aparecerem os botoes, "false" para não aparecerem
		 * 
		 * @param	carrinho				"true" para aparecerem o carrinho_mc e o trilho_mc, "false" para não aparecerem
		 * 
		 * 
		 * @langversion ActionScript 3.0
		 * @playerversion Flash 9.0
		 */
		public function ScrollVerticalFB (mascara:Sprite, container:Sprite, movimento:Number = 0, redimensionamento:Boolean = false, delay:Number = .1, botoes:Boolean = true, carrinho:Boolean = true):void {
			_mascara = mascara;
			_container = container;
			
			//_movimento no fim da função
			_delay = delay;
			_redimensionamento = redimensionamento;
			_botoes = botoes;
			_carrinho = carrinho;
			

			pontoMaximo = _mascara.y;
			pontoMinimo = _mascara.y + _mascara.height - _container.height;
			
			_container.mask = _mascara;
			
			scroll = new mcScrollVertical();
			
			
			if (!_botoes && !_carrinho) {
				_botoes = _carrinho = true;
			}
			
			if (!_botoes) {
				scroll.baixo_mc.visible = false;
				scroll.cima_mc.visible = false;
				scroll.trilho_mc.height = _mascara.height;
				scroll.trilho_mc.y = 0;
				scroll.carrinho_mc.y = 0;
			} else {
				scroll.trilho_mc.height = mascara.height - (scroll.cima_mc.height + 4) - (scroll.baixo_mc.height + 4);
			}
			
			
			if (!_carrinho) {
				scroll.carrinho_mc.visible = false;
				scroll.trilho_mc.visible = false;
			}
			
			
			scroll.y = mascara.y;
			scroll.x = mascara.x+mascara.width+4;
			scroll.baixo_mc.y = mascara.height - scroll.baixo_mc.height;
			
			if (_redimensionamento) {
				scroll.carrinho_mc.height = _mascara.height / _container.height * scroll.trilho_mc.height;
				trace("redimensionou o carrinho vertical");
				trace(scroll.carrinho_mc.height );
			}
			limiteArraste = new Rectangle(scroll.carrinho_mc.x, scroll.carrinho_mc.y, 0, scroll.trilho_mc.height-scroll.carrinho_mc.height+.5)
			configuraAcoes();
			
			_movimento = (movimento == 0)? _mascara.height / _container.height * scroll.trilho_mc.height : movimento;
			
			destinoContainer = _mascara.y;
			destinoCarrinho = scroll.trilho_mc.y;
			
			addChild(scroll);
		}
		
		private function configuraAcoes ():void {
			scroll.baixo_mc.gotoAndStop(1);
			scroll.cima_mc.gotoAndStop(1);
			scroll.carrinho_mc.gotoAndStop(1);
			
			scroll.baixo_mc.buttonMode = true;
			scroll.cima_mc.buttonMode = true;
			scroll.carrinho_mc.buttonMode = true;
			
			scroll.baixo_mc.addEventListener(MouseEvent.MOUSE_OVER, over);
			scroll.cima_mc.addEventListener(MouseEvent.MOUSE_OVER, over);
			scroll.carrinho_mc.addEventListener(MouseEvent.MOUSE_OVER, over);
			
			scroll.baixo_mc.addEventListener(MouseEvent.MOUSE_OUT, out);
			scroll.cima_mc.addEventListener(MouseEvent.MOUSE_OUT, out);
			scroll.carrinho_mc.addEventListener(MouseEvent.MOUSE_OUT, out);
			
			
			//descendo
			scroll.baixo_mc.addEventListener(MouseEvent.MOUSE_DOWN, function ():void {
				vaiPraBaixo ();
				intervaloBotoes = setInterval(vaiPraBaixo, 300);
				_container["root"].addEventListener(MouseEvent.MOUSE_UP, function (evt:MouseEvent):void {
					clearInterval(intervaloBotoes);
				});
			});
			
			//subindo
			scroll.cima_mc.addEventListener(MouseEvent.MOUSE_DOWN, function ():void {
				vaiPraCima ();
				intervaloBotoes = setInterval(vaiPraCima, 300);
				_container["root"].addEventListener(MouseEvent.MOUSE_UP, function (evt:MouseEvent):void {
					clearInterval(intervaloBotoes);
				});
			});
			
			scroll.carrinho_mc.addEventListener(MouseEvent.MOUSE_DOWN, function (evt:MouseEvent):void {
				scroll.carrinho_mc.startDrag(false, limiteArraste);
				_container["root"].addEventListener(MouseEvent.MOUSE_UP, function (evt:MouseEvent):void { 
					destinoCarrinho = scroll.carrinho_mc.y;
					scroll.carrinho_mc.stopDrag();
					_container["root"].removeEventListener(Event.ENTER_FRAME, atualizaCarrinhoArrastando);
				} );
				_container["root"].addEventListener(Event.ENTER_FRAME, atualizaCarrinhoArrastando);
			});			
		}
		
		private function vaiPraBaixo ():void {
			if (destinoCarrinho + _movimento < scroll.trilho_mc.y + scroll.trilho_mc.height-scroll.carrinho_mc.height-.1) {
				destinoCarrinho = Math.floor(destinoCarrinho + _movimento);
				trace("destinoCarrinho" + destinoCarrinho);
			} else {
				destinoCarrinho = scroll.trilho_mc.y+scroll.trilho_mc.height-scroll.carrinho_mc.height-.1;
			}
			atualizaCarrinho();
			
			TweenLite.to(scroll.carrinho_mc, _delay, { y:destinoCarrinho, onUpdate:botaoPraBaixo } );
		}
		private function vaiPraCima ():void {
			if (destinoCarrinho - _movimento > scroll.trilho_mc.y) {
				destinoCarrinho = Math.floor(destinoCarrinho - _movimento);
			} else {
				destinoCarrinho = scroll.trilho_mc.y;
			}
			atualizaCarrinho();
			
			TweenLite.to(scroll.carrinho_mc, _delay, { y:destinoCarrinho, onUpdate:botaoPraBaixo } );
		}
		
		private function botaoPraBaixo (evt:Event = null):void {
			atualizaCarrinho();
		}
		
		private function atualizaCarrinho(evt:Event = null):void {
			var fator:Number = (scroll.carrinho_mc.y - scroll.trilho_mc.y) / (scroll.trilho_mc.height - scroll.carrinho_mc.height);
			destinoContainer = pontoMaximo - (pontoMaximo - pontoMinimo) * fator;
			_container.y = destinoContainer;
		}
		
		private function atualizaCarrinhoArrastando(evt:Event = null):void {
			var fator:Number = (scroll.carrinho_mc.y - scroll.trilho_mc.y) / (scroll.trilho_mc.height - scroll.carrinho_mc.height);
			destinoContainer = pontoMaximo - (pontoMaximo - pontoMinimo) * fator;
			TweenLite.to(_container, _delay, { y:destinoContainer } );
		}
		
		private function over (evt:MouseEvent):void {
			evt.currentTarget.gotoAndStop(2);
		}
		private function out (evt:MouseEvent):void {
			evt.currentTarget.gotoAndStop(1);
		}
		
		
		/**
		 * [read-only] No caso de mudanças nas dimensões do container chamar essa função.
		 * 
		 * @langversion ActionScript 3.0
		 * @playerversion Flash 9.0
		 
		public function recarrega():void {
			
			pontoMaximo = _mascara.y;
			pontoMinimo = _mascara.y + _mascara.height - _container.height;
			
			_movimento = 
			
			scroll.baixo_mc.y = _mascara.height - scroll.baixo_mc.height;
			
			if (_redimensionamento) {
				scroll.carrinho_mc.height = _mascara.height / _container.height * scroll.trilho_mc.height;
			}
			
			
			
			destinoContainer = _mascara.y;
			destinoCarrinho = scroll.trilho_mc.y;
			scroll.carrinho_mc.y = scroll.trilho_mc.y;
			limiteArraste = new Rectangle(scroll.carrinho_mc.x, scroll.carrinho_mc.y, 0, scroll.trilho_mc.height-scroll.carrinho_mc.height+.5)
			configuraAcoes();
			
			_movimento = (_movimento == 0)? _mascara.height / _container.height * scroll.trilho_mc.height : _movimento;
			
			atualizaCarrinhoArrastando();
		}*/
	}
}